@@ -38,6 +38,9 @@ struct microstep {
 struct macrostep {
     struct edge *edge;              // the edge this macrostep corresponds to
     unsigned int tid;               // thread identifier
+#ifdef notdef
+    unsigned int tid2;               // thread identifier
+#endif
     hvalue_t name, arg;             // name and argument of the thread (first method)
     struct callstack *cs;           // callstack of the thread at the beginning
 
@@ -88,9 +91,13 @@ struct phase {
 // are used across multiple modules.
 struct global {
     struct code code;               // code of the Harmony program
+    struct json_value *locs;        // from hvm file
+    struct json_value *modules;     // from hvm file
+    struct json_value *pretty;      // from hvm file
     struct dict *values;            // dictionary of values
     struct dict *computations;      // evaluated Harmony byte code
-    hvalue_t seqs;                  // sequential variables
+    hvalue_t *seqs;                 // sequential variables
+    unsigned int nseqs;             // number of sequential variables
     struct worker *workers;         // points to array of workers
     unsigned int nworkers;          // total number of workers
     struct phase phases[MAX_PHASES];// for reporting
@@ -106,10 +113,12 @@ struct global {
     //  2: look up states and add to the todo list if new
     //  3: resize tables
     // The barriers are to synchronize these three phases.
-    barrier_t start_barrier, middle_barrier, end_barrier;
+    barrier_t barrier;
 
     // invariants
+#ifdef OBSOLETE
     mutex_t inv_lock;               // lock on list of invariants and finals
+#endif
     unsigned int ninvs;             // number of invariants
     struct invariant *invs;         // list of invariants
     bool inv_pre;                   // some invariant uses "pre"
@@ -117,16 +126,14 @@ struct global {
     hvalue_t *finals;               // contexts of finally preds
 
     struct graph graph;             // the Kripke structure but also the todo list
+    struct node *initial;           // initial node
     uint8_t *neps;                  // #epsilon edges for each node
     bool printed_something;         // set if anything was printed
     struct failure *failures;       // queue of "struct failure"  (TODO: make part of struct node "issues")
     hvalue_t *processes;            // array of contexts of processes
     struct callstack **callstacks;  // array of callstacks of processes
     unsigned int nprocesses;        // the number of processes in the list
-    double lasttime;                // since last report printed
-    unsigned int last_nstates;      // to measure #states / second
     struct dfa *dfa;                // for tracking correct behaviors
-    struct json_value *pretty;      // for output
     bool run_direct;                // non-model-checked mode
     unsigned long allocated;        // allocated table space
     unsigned int oldpid;            // for thread id computation
@@ -139,7 +146,7 @@ struct global {
     unsigned int nsymbols;
 
     // Reconstructed error trace stored here
-    unsigned int nmacrosteps, alloc_macrosteps;
+    unsigned int nmacrosteps;
     struct macrostep **macrosteps;
 };
 
