@@ -32,12 +32,20 @@ void panic(char *s){
     exit(1);
 }
 
+
 #ifdef __APPLE__
+#ifdef __clang__
 void *my_aligned_alloc(size_t alignment, size_t size){
     if (__builtin_available(macOS 10.15, *)) {
         return aligned_alloc(alignment, size);
     }
-    panic("aligned_alloc not available in current version of MacOSX");
-    return NULL;
+
+    static bool warned;
+    if (!warned) {
+        fprintf(stderr, "aligned_alloc not available in current version of MacOSX\n");
+        warned = true;
+    }
+    return malloc(size);
 }
 #endif
+#endif
