@@ -17,9 +17,8 @@
 #define HEAP_ALLOC
 #else // _WIN32
 #define ALIGNED_ALLOC
-
 #include <stdlib.h>
-#ifdef __APPLE__
+#if defined(__APPLE__) && defined(__clang__)
 void *my_aligned_alloc(size_t alignment, size_t size);
 #else
 #define my_aligned_alloc(a, s)  aligned_alloc(a, s)
