@@ -171,7 +171,7 @@ static inline void *value_get(hvalue_t v, unsigned int *psize){
         }
         return NULL;
     }
-    return dict_retrieve((void *) v, psize);
+    return dict_retrieve(HV_TO_P(v), psize);
 }
 
 bool value_trystore(struct allocator *allocator, hvalue_t dict, hvalue_t key, hvalue_t value, bool allow_inserts, bool *is_append, hvalue_t *result);
@@ -193,7 +193,7 @@ void context_remove(struct state *state, hvalue_t ctx);
 int context_add(struct state *state, hvalue_t ctx);
 int stopped_context_add(struct state *state, hvalue_t ctx);
 char *json_escape_value(hvalue_t v);
-void value_trace(FILE *file, struct callstack *cs, unsigned int pc, hvalue_t vars, char *prefix);
+void value_trace(FILE *file, struct callstack *cs, unsigned int pc, hvalue_t vars);
 void print_vars(FILE *file, hvalue_t v);
 
 #endif //SRC_VALUE_H
