@@ -39,12 +39,20 @@ class AST:
     def stmt(self):
         _, _, line1, column1 = self.token
         lexeme2, _, line2, column2 = self.endtoken
-        return (line1, column1, line2, column2 + len(lexeme2) - 1)
+        # Hack: some piece of external software puts the lexeme 'newLine' in there sometimes
+        if lexeme2 == "newLine" and column2 == 1:
+            return (line1, column1, line2, column2)
+        else:
+            return (line1, column1, line2, column2 + len(lexeme2) - 1)
 
     def range(self, token1, token2):
         _, _, line1, column1 = token1
         lexeme2, _, line2, column2 = token2
-        return (line1, column1, line2, column2 + len(lexeme2) - 1)
+        # Hack: some piece of external software puts the lexeme 'newLine' in there sometimes
+        if lexeme2 == "newLine" and column2 == 1:
+            return (line1, column1, line2, column2)
+        else:
+            return (line1, column1, line2, column2 + len(lexeme2) - 1)
 
     # a new local constant or tree of constants
     def define(self, scope: Scope, const):
@@ -371,7 +379,15 @@ class NameAST(AST):
             else:
                 code.append(StoreVarOp(None, self.name[0]), start, stop, stmt=stmt)
         else:
-            assert t == "global", (t, v)
+            if t != "global":
+                (lexeme, file, line, column) = v
+                raise HarmonyCompilerError(
+                    message="not a mutable variable: " + lexeme,
+                    lexeme=lexeme,
+                    filename=file,
+                    line=line,
+                    column=column
+                )
             code.append(StoreOp(None, self.name, None), start, stop, stmt=stmt)
 
     def isConstant(self, scope):
@@ -1791,10 +1807,11 @@ class MethodAST(AST):
         self.stat.compile(ns, code, stmt)
         if self.atomically:
             code.append(AtomicDecOp(), self.token, self.endtoken, stmt=stmt)
+        stmt = self.range(self.endtoken, self.endtoken)
         if self.result is None:
-            code.append(ReturnOp(result, AddressValue(None, [])), self.token, self.endtoken, stmt=stmt)
+            code.append(ReturnOp(result, AddressValue(None, [])), self.endtoken, self.endtoken, stmt=stmt)
         else:
-            code.append(ReturnOp(result, None), self.token, self.endtoken, stmt=stmt)
+            code.append(ReturnOp(result, None), self.endtoken, self.endtoken, stmt=stmt)
         code.nextLabel(endlabel)
 
         # promote global variables
@@ -2137,9 +2154,7 @@ class SequentialAST(AST):
 
     def compile(self, scope, code, stmt):
         for var in self.vars:
-            assert isinstance(var, NameAST)
-            (t, v) = scope.lookup(var.name)
-            lexeme, file, line, column = var.name
+            (t, v) = scope.lookup(var)
             if t != "global":
                 raise HarmonyCompilerError(
                     filename=file,
@@ -2149,9 +2164,9 @@ class SequentialAST(AST):
                     message="%s: Parse error: shadows prior definition" % lexeme
                 )
         stmt = self.stmt()
-        for lv in self.vars:
-            lv.ph1(scope, code, stmt)
-            code.append(SequentialOp(), self.token, self.endtoken, stmt=stmt)
+        for var in self.vars:
+            # OBSOLETE var.ph1(scope, code, stmt)
+            code.append(SequentialOp(var), self.token, self.endtoken, stmt=stmt)
 
     def getLabels(self):
         return set()
