@@ -3,6 +3,7 @@ import json
 import pathlib
 import webbrowser
 import sys
+import os
 import argparse
 
 from antlr4 import * # type: ignore
@@ -22,6 +23,10 @@ args = argparse.ArgumentParser(
     "harmony", description="Harmony programming language compiler and model checker")
 args.add_argument("-a", action="store_true",
                   help="list machine code (with labels)")
+args.add_argument("-b", action="store_true",
+                  help="enable behavior subset check")
+args.add_argument("-T", action="store_true",
+                  help="print timing info")
 args.add_argument("-A", action="store_true",
                   help="list machine code (without labels)")
 args.add_argument("-B", type=str, help="check against the given behavior")
@@ -33,16 +38,22 @@ args.add_argument("-c", "--const", action='append', type=str,
                   metavar="name=value", help="define a constant")
 args.add_argument("-D", action="store_true",
                   help="dump Kripke graph")
+args.add_argument("-W", action="store_true",
+                  help="suppress busy waiting check")
 args.add_argument("-R", action="store_true",
                   help="suppress race condition warnings")
+args.add_argument("-U", action="store_true",
+                  help="do not pin workers")
 args.add_argument("--module", "-m", action="append", type=str,
                   metavar="module=version", help="select a module version")
 args.add_argument("-i", "--intf", type=str, metavar="expr",
-                  help="specify in interface function")
+                  help="specify an interface function")
 args.add_argument("-s", action="store_true",
                   help="silent (do not print periodic status updates)")
 args.add_argument("-v", "--version", action="store_true",
                   help="print version number")
+args.add_argument("--quick", action="store_true",
+                  help="no post-model-check analysis")
 args.add_argument("-o", action='append', type=pathlib.Path,
                   help="specify output file (.hvm, .hco, .hfa, .htm. .tla, .tex, .png, .gv)")
 args.add_argument("-j", action="store_true",
@@ -62,7 +73,14 @@ args.add_argument("--cf", action="append", type=str, help=argparse.SUPPRESS)
 args.add_argument("args", metavar="args", type=str, nargs='*', help="arguments")
 
 def handle_hny(ns, output_files, parse_code_only, filenames):
-    print("Phase 1: compile Harmony program to bytecode")
+    for suffix, file in output_files.items():
+        if file is not None:
+            try:
+                os.remove(file)
+            except:
+                pass
+
+    print("* Phase 1: compile Harmony program to bytecode", flush=True)
 
     consts: List[str] = ns.const or []
     interface: Optional[str] = ns.intf
@@ -105,12 +123,20 @@ def handle_hvm(ns, output_files, parse_code_only, code, scope):
     charm_options = ns.cf or []
     if ns.B:
         charm_options.append("-B" + ns.B)
+    if ns.b:
+        charm_options.append("-b")
+    if ns.T:
+        charm_options.append("-T")
     if ns.w:
         charm_options.append("-w" + ns.w)
     if ns.D:
         charm_options.append("-D")
     if ns.R:
         charm_options.append("-R")
+    if ns.U:
+        charm_options.append("-U")
+    if ns.W:
+        charm_options.append("-c")
 
     # see if there is a configuration file
     if code is not None:
@@ -121,7 +147,8 @@ def handle_hvm(ns, output_files, parse_code_only, code, scope):
         exit()
 
     if ns.direct:
-        print("Phase 2: run", flush=True)
+        # print("* Phase 2: run", flush=True)
+        charm_options.append("-d")
         r = charm.run_model_checker(
             *charm_options,
             output_files["hvm"]
@@ -130,17 +157,28 @@ def handle_hvm(ns, output_files, parse_code_only, code, scope):
             print("charm failed")
             exit(r)
     else:
-        print("Phase 2: run the model checker", flush=True)
-        r = charm.run_model_checker(
-            *charm_options,
-            "-o" + output_files["hco"],
-            output_files["hvm"]
-        )
+        # print("* Phase 2: run the model checker", flush=True)
+        if "hfa" in output_files and output_files["hfa"] != None:
+            r = charm.run_model_checker(
+                *charm_options,
+                "-o" + output_files["hco"],
+                "-o" + output_files["hfa"],
+                output_files["hvm"]
+            )
+        else:
+            r = charm.run_model_checker(
+                *charm_options,
+                "-o" + output_files["hco"],
+                output_files["hvm"]
+            )
         if r != 0:
             print("charm model checker failed")
             exit(r)
 
 def handle_hco(ns, output_files):
+    if ns.quick:
+        return
+
     suppress_output = ns.suppress
 
     behavior = None
@@ -152,15 +190,20 @@ def handle_hco(ns, output_files):
     b = Brief()
     b.run(output_files, behavior)
     vb = Verbose()
-    vb.run(output_files, behavior)
+    vb.run(output_files)
     gh = GenHTML()
     gh.run(output_files)
     if not suppress_output:
+        print()
         p = pathlib.Path(output_files["htm"]).resolve()
         url = "file://" + str(p)
-        print("open " + url + " for more information", file=sys.stderr)
+        print("open " + url + " for detailed information", file=sys.stderr)
         if not disable_browser:
             webbrowser.open(url)
+    # exit as minify may still be running
+    print(flush=True, end="")
+    print(file=sys.stderr, flush=True, end="")
+    os._exit(0)
     
 def handle_version(_: argparse.Namespace):
     print("Version:", harmony_model_checker.__package__,
@@ -248,6 +291,10 @@ def main():
         output_files["htm"] = stem + ".htm"
     if output_files["hvb"] is None:
         output_files["hvb"] = stem + ".hvb"
+    if output_files["hfa"] is None:
+        output_files["hfa"] = stem + ".hfa"
+    if output_files["png"] is None:
+        output_files["png"] = stem + ".png"
     if output_files["png"] is not None and output_files["gv"] is None:
         output_files["gv"] = stem + ".gv"
 
@@ -270,14 +317,14 @@ def main():
             handle_hvm(ns, output_files, parse_code_only, code, scope)
             handle_hco(ns, output_files)
         else:
-            print("Skipping Phases 2-5...")
+            print("Skipping Phases 2-5...", flush=True)
             legacy_harmony.dumpCode(print_code, code, scope)
 
     if input_file_type == ".hvm":
-        print("Skipping Phase 1...")
+        print("Skipping Phase 1...", flush=True)
         handle_hvm(ns, output_files, parse_code_only, None, None)
         handle_hco(ns, output_files)
         
     if input_file_type == ".hco":
-        print("Skipping Phases 1-4...")
+        print("Skipping Phases 1-4...", flush=True)
         handle_hco(ns, output_files)
