@@ -58,6 +58,9 @@ class HarmonyVisitorImpl(HarmonyVisitor):
     
     def visitImport_names_seq(self, ctx: HarmonyParser.Import_names_seqContext):
         return [self.get_token(d.symbol, str(d)) for d in ctx.NAME()]
+    
+    def visitSequential_names_seq(self, ctx: HarmonyParser.Sequential_names_seqContext):
+        return [self.get_token(d.symbol, str(d)) for d in ctx.NAME()]
 
     # Visit a parse tree produced by HarmonyParser#program.
     def visitProgram(self, ctx: HarmonyParser.ProgramContext):
@@ -315,7 +318,7 @@ class HarmonyVisitorImpl(HarmonyVisitor):
 
     # Visit a parse tree produced by HarmonyParser#sequential_stmt.
     def visitSequential_stmt(self, ctx: HarmonyParser.Sequential_stmtContext):
-        expr = [self.visit(e) for e in ctx.expr()]
+        expr = self.visit(ctx.sequential_names_seq())
         tkn = self.get_token(ctx.start, ctx.start.text)
         endtoken = self.get_token(ctx.stop, ctx.stop.text)
         return SequentialAST(endtoken, tkn, False, expr)
@@ -670,6 +673,7 @@ class HarmonyVisitorImpl(HarmonyVisitor):
         for o in ops:
             if o[0] in { '/', '//', '%', 'mod' }:
                 if found:
+                    token = ops[0]
                     raise HarmonyCompilerError(
                         message="Expression too complicated: use parentheses",
                         filename=self.file,
@@ -719,6 +723,7 @@ class HarmonyVisitorImpl(HarmonyVisitor):
         if all(o[0] in { '+', '-', '*', '/', '//', '%', 'mod' } for o in ops):
             return self.simple_expr(ops, exprs)
 
+        token = ops[0]
         raise HarmonyCompilerError(
             message="Expression too complicated: use parentheses",
             filename=self.file,
@@ -772,7 +777,7 @@ class HarmonyVisitorImpl(HarmonyVisitor):
         bool_ops = [ o[0] for o in ops if o[0] in { 'or', 'and', '=>', '==', '=' } ]
         if 'or' in bool_ops:
             if len(set(bool_ops)) > 1:
-                assert False, bool_ops
+                token = ops[0]
                 raise HarmonyCompilerError(
                     message="Boolean expression too complicated: use parentheses",
                     filename=self.file,
@@ -783,6 +788,7 @@ class HarmonyVisitorImpl(HarmonyVisitor):
             return self.bool_expr_helper(ops, exprs, 'or')
         if 'and' in bool_ops:
             if len(set(bool_ops)) > 1:
+                token = ops[0]
                 raise HarmonyCompilerError(
                     message="Boolean expression too complicated: use parentheses",
                     filename=self.file,
@@ -793,6 +799,7 @@ class HarmonyVisitorImpl(HarmonyVisitor):
             return self.bool_expr_helper(ops, exprs, 'and')
         if '=>' in bool_ops:
             if len(bool_ops) > 1:
+                token = ops[0]
                 raise HarmonyCompilerError(
                     message="Boolean imply expression too complicated: use parentheses",
                     filename=self.file,
